# Changelog

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)
and this project adheres to [Calendar Versioning](https://calver.org/#scheme)
with the `YY.0M.MICRO` scheme.

<img src="https://img.shields.io/badge/calver-YY.0M.MICRO-22bfda.svg">

## Unreleased

### Features

- Allow specifying `picodata connect [user@][host][:port]` format. It
  overrides the `--user` option.

- _Clusterwide SQL_ now uses an internal module called `key_def` to
  determine tuple buckets. In case the spaces were sharded using a
  different hash function, executing SQL queries on these spaces would
  return inaccurate outcomes. For more examples, refer to
  `pico.help('create_space')`.

- _Clusterwide SQL_ now features Lua documentation. Refer to
  `pico.help('sql')` for more information.

- _Clusterwide SQL_ now enables the creation of sharded tables.
  To learn more, please consult `pico.help('sql')`.

- _Clusterwide SQL_ introduces the capability to delete sharded tables.
  To obtain more details, please consult `pico.help('sql')`.

### Lua API:

- Support for user password control policies.
- Update `pico.LUA_API_VERSION`: `1.0.0` -> `1.3.0`
- Add `pico.raft_term()`
- Add `pico.create_user()`
- Add `pico.drop_user()`
- Add `pico.change_password()`
- Add `pico.grant_privilege()`
- Add `pico.revoke_privilege()`
- Add `pico.drop_space()`
- Add `pico.wait_ddl_finalize()`
- Add `pico.create_role()`
- Add `pico.drop_role()`
- Add `pico.create_password_policy()`
- Add `pico.set_password_policy()`
- Add `pico.drop_password_policy()`

## [23.06.0] - 2023-06-16

### Features

- Expose the Public Lua API `pico.*`. It's supplemented with a
  built-in reference manual, see `pico.help()`.

- _Compare and Swap_ (CaS) is an algorithm that allows to combine Raft
  operations with a predicate checking. It makes read-write access to
  global spaces serializable. See `pico.help('cas')`.

- _Clusterwide schema_ now allows to create global and sharded spaces.
  The content of global spaces is replicated to every instance in the
  cluster. In sharded spaces the chuncs of data (buckets) are
  distributed across different replicasets. See
  `pico.help('create_space')`.

- _Raft log compaction_ allows stripping old raft log entries in order
  to prevent its infinite growth. See `pico.help('raft_compact_log')`.

- Remove everything related to "migrations". They're superseeded with
  "clusterwide schema" mentioned above.

### CLI

- `picodata run --script` command-line argument sets a path to a Lua
  script executed at startup.

- `picodata run --http-listen` command-line argument sets the [HTTP
  server](https://github.com/tarantool/http) listening address. If no
  value is provided, the server won't be initialized.

- `picodata connect` CLI command provides interactive Lua console.

### Implementation details

- Picodata automatically demotes Raft voters that go offline and
  promotes a replacement. See docs/topology.md for more details.
  Replicaset leadership is switched too.

### Compatibility

- The current version is NOT compatible with prior releases. It cannot
  be started with the old snapshots.

## [22.11.0] - 2022-11-22

### Features

- Brand new algorithm of cluster management based on the _"governor"_
  concept — a centralized actor that maintains cluster topology and
  performs instances configuration.

- Instance states are now called _"grades"_. This new term more clearly
  denotes how an instance is currently perceived by other instances (eg.
  how they are configured in its regard) rather than what it assumes
  about itself.

- Built-in _sharding_ configuration based on the `vshard` library. Once
  a replicaset is up to the given replication factor, Picodata will
  automatically re-balance data across replicasets.

- Clusterwide schema and data _migrations_ are introduced.

- Instances can now be _expelled_ in order to shrink the cluster.

### Compatibility

- The current version is NOT compatible with `22.07.0`. It cannot be
  started with the old snapshots.

## [22.07.0] - 2022-07-08

### Basic functionality

- Command line interface for cluster deployment.
- Dynamic topology configuration, cluster scaling by launching more instances.
- Support for setting the replication factor for the whole cluster.
- Failure domains-aware replicasets composition.
- Two kinds of storages:
  - based on Raft consensus algorithm (clusterwide),
  - based on Tarantool master-master async replication.
- Graceful instance shutdown.
- Automatic Raft group management (voters provision, Raft leader failover).
- Dead instance rebootstrap without data loss.
- Automatic peers discovery during initial cluster configuration.
