# Топология кластера Picodata

В данном документе рассматриваются различные сценарии работы с кластером. Все они основаны на одном и том же принципе: запуске и объединении отдельных экземпляров Picodata в распределенный кластер. При этом сложность развертывания и поддержания работоспособности кластера зависит только от сложности его топологии.

[TOC]

---

## Минимальный вариант кластера

Picodata может создать кластер, состоящий всего из одного экземпляра/инстанса. Обязательных параметров у него нет, что позволяет свести запуск к выполнению всего одной простой команды:

```
picodata run
```

Можно добавлять сколько угодно последующих инcтансов — все они будут подключаться к этому кластеру. Каждому интансу следует задать отдельную рабочую директорию (параметр `--data-dir`), а также указать адрес и порт для приема соединений (параметр `--listen`) в формате `<HOST>:<PORT>`. Фактор репликации по умолчанию равен 1 — каждый инстанс образует отдельный репликасет. Если для `--listen` указать только порт, то будет использован IP-адрес по умолчанию (127.0.0.1):

```
picodata run --data-dir i1 --listen :3301
picodata run --data-dir i2 --listen :3302
picodata run --data-dir i3 --listen :3303
```

## Кластер на нескольких серверах

Выше был показан запуск Picodata на одном сервере, что удобно для тестирования и отладки, но не отражает сценариев полноценного использования кластера. Поэтому пора запустить Picodata на нескольких серверах. Предположим, что их два: `192.168.0.1` и `192.168.0.2`. Порядок запуска будет следующим:

На `192.168.0.1`:
```shell
picodata run --listen 192.168.0.1:3301
```

На `192.168.0.2`:
```shell
picodata run --listen 192.168.0.2:3301 --peer 192.168.0.1:3301
```

На что нужно обратить внимание:

Во-первых, для параметра `--listen` вместо стандартного значения `127.0.0.1` надо указать конкретный адрес. Формат адреса допускает упрощения — можно указать только хост `192.168.0.1` (порт по умолчанию `:3301`), или только порт, но для наглядности лучше использовать полный формат `<HOST>:<PORT>`.

Значение параметра `--listen` не хранится в кластерной конфигурации и может меняться при перезапуске инстанса.

Во-вторых, надо дать инстансам возможность обнаружить друг друга для того чтобы механизм [discovery](discovery.md) правильно собрал все найденные экземпляры Picodata в один кластер. Для этого в параметре `--peer` нужно указать адрес какого-либо соседнего инстанса. По умолчанию значение параметра `--peer` установлено в `127.0.0.1:3301`. Параметр `--peer` не влияет больше ни на что, кроме механизма обнаружения других инстансов.

Параметр `--advertise` используется для установки публичного IP-адреса и порта инстанса. Параметр сообщает, по какому адресу остальные инстансы должны обращаться к текущему. По умолчанию он равен `--listen`, поэтому в примере выше не упоминается. Но, например, в случае `--listen 0.0.0.0` его придется указать явно:

```shell
picodata run --listen 0.0.0.0:3301 --advertise 192.168.0.1:3301
```

Значение параметра `--advertise` анонсируется кластеру при запуске инстанса. Его можно поменять при перезапуске инстанса или в процессе его работы командой `picodata set-advertise`.

## Питомцы против стада

Чтобы проще было отличать инстансы друг от друга, им можно давать имена:

```
picodata run --instance-id barsik
```

Если имя не дать, то оно будет сгенерировано автоматически в момент добавления в кластер. Имя инстанса задается один раз и не может быть изменено в дальнейшем (например, оно постоянно сохраняется в снапшотах инстанса). В кластере нельзя иметь два инстанса с одинаковым именем — второй инстанс сразу после запуска получит ошибку при добавлении в кластер. Тем не менее, имя можно повторно использовать если предварительно исключить первый инстанс с таким именем из кластера. Это делается командой `picodata expel barsik`.

## Группы и роли

До сих пор рассматриваемый кластер был гомогенным. Все инстансы были одинаковы по функциональности — хранили данные, обрабатывали запросы. В промышленной эксплуатации эти роли почти всегда требуется разделять, чтобы эффективнее использовать ресурсы оборудования. Под хранение выделяются серверы с большим объемом памяти, для обработки запросов это не требуется.

В Picodata для этих целей служит понятие групп инстансов. Принадлежность инстанса той или иной группе задается при добавлении в кластер параметром `--group` и впоследствии не может быть изменена. По умолчанию кластер состоит из одной группы "common".

Функциональность инстансов определяется набором ролей. На данный момент существует две роли:

- storage — позволяет хранить шардированные данные на инстансе.
- router — реализует логику доступа к шардированным данным.

По умолчанию инстанс исполняет обе роли одновременно, но его можно ограничить явным указанием одной из них:

```
picodata run --role storage
picodata run --role router
```

Важно то, что обе эти роли относятся только к шардированию. Так, отключение роли storage ничем не мешает хранить данные локально.

Также инстанс можно запустить без ролей вовсе, в результате чего он будет функционировать исключительно как не-шардированное локальное хранилище:

```
picodata run --no-role
```

Все инстансы в группе имеют одинаковый набор ролей и одинаковый фактор репликации.

## Репликация и зоны доступности (failure domains)

Количество инстансов в репликасете определяется значением переменной `replication_factor`. Внутри группы инстансов используется один и тот же `replication_factor`.

Для ее инициализации служит параметр `--init-replication-factor`. Этот параметр играет роль только в момент создания группы (добавления первого инстанса). В этот момент значение из аргументов командной строки записывается в конфигурацию кластера. В дальнейшем значение параметра `--init-replication-factor` игнорируется.

Отредактировать фактор репликации, сохраненный в конфигурации кластера, можно командой `picodata set-replication-factor`. Редактирование конфигурации сказывается только на вновь добавляемых инстансах, но не затрагивает уже работающие.

По мере усложнения топологии возникает еще один вопрос — как не допустить объединения в репликасет инстансов из одного и того же датацентра. Для этого введен параметр `--failure-domain` — _зона доступности_, отражающая признак физического размещения сервера, на котором выполняется инстанс Picodata. Это может быть как датацентр, так и какое-либо другое обозначение расположения: регион (например, `eu-east`), стойка, сервер, или собственное обозначение (blue, green, yellow). Ниже показан пример запуска инстанса Picodata с указанием зоны доступности:

```
picodata run --init-replication-factor 2 --failure-domain region=us,zone=us-west-1
```

Добавление инстанса в репликасет происходит по следующим правилам:

- Если в каком-либо репликасете количество инстансов меньше необходимого фактора репликации, то новый инстанс добавляется в него при условии, что их параметры `--failure-domain` отличаются (регистр символов не учитывается).
- Если подходящих репликасетов нет, то Picodata создает новый репликасет.

Параметр `--failure-domain` играет роль только в момент добавления инстанса в кластер. **Принадлежность инстанса репликасету впоследствии не меняется**.

Как и параметр `--advertise`, значение параметра`--failure-domain` каждого инстанса можно редактировать:

- Либо перезапустив инстанс с новыми параметрами.
- Либо в процессе его работы командой `picodata set-failure-domain`.

Добавляемый инстанс должен обладать тем же набором параметров, которые уже есть в кластере. Например, инстанс `dc=msk` не сможет присоединиться к кластеру с `--failure-domain region=eu/us` и вернет ошибку.

Как было указано выше, сравнение зон доступности производится без учета
регистра символов, поэтому, к примеру, два инстанса с аргументами
`--failure-domain region=us` и `--failure-domain REGION=US` будут относиться
к одному региону и, следовательно, не попадут в один репликасет (исключения
описаны ниже).

## Кейс: два датацентра по две реплики

Picodata старается не объединять в один репликасет инстансы, у которых совпадает хотя бы один домен. Но иногда это все же необходимо. Чтобы ограничить Picodata в бесконечном создании репликасетов, можно воспользоваться флагом `--max-replicaset-count` (по умолчанию `inf`).

Как и `--init-replication-factor`, параметр `--max-replicaset-count` может быть разным для разных групп.

Как и другие параметры, `--max-replicaset-count` редактируется в любой момент:

- При добавлении нового инстанса
- В процессе его работы командой `picodata set-max-replicaset-count`

Важно учитывать, что параметр `--max-replicaset-count` нельзя сделать меньше существующего количества репликасетов.

## Файлы конфигурации

Существует три способа передать Picodata параметры конфигурации. Они приведены ниже в порядке возрастания приоритета:

1. Файл конфигурации (yaml / toml)
2. Переменные окружения `PICODATA_<PARAM>=<VALUE>`
3. Аргументы командной строки `--param value`

Мы перечислили достаточно много разнобразных параметров, некоторые из которых делают команду запуска достаточно длинной. Вместо отдельных команд можно использовать файл конифгурации. Пример:

<h5 a><strong><code>storage.toml</code></strong></h5>

```toml
group = "storages"
max-replicaset-count = 30
replication-factor = 4
roles = ["storage"]
```

<h5 a><strong><code>storage.toml</code></strong></h5>

```toml
group = "routers"
roles = ["router"]
```

Пример запуска кластера Picodata c использованием файла конфигурации:

```
picodata run --cfg storage.toml --listen :3301
picodata run --cfg router.toml --listen :3302
```

## Динамическое переключение голосующих узлов в Raft (Raft voter failover)

Все узлы Raft в кластере делятся на два типа: голосующие (`voter`) и неголосующие (`learner`). За консистентность Raft-группы отвечают только узлы первого типа. Для коммита каждой транзакции требуется собрать кворум из `N/2 + 1` из голосующих узлов. Неголосующие узлы в кворуме не участвуют.

Чтобы сохранить баланс между надежностью кластера и удобством его эксплуатации, в Picodata предусмотрена удобная функция — динамическое переключение типа узлов. Если один из голосующих узлов становится недоступен или прекращает работу (что может нарушить кворум в Raft), то тип `voter` автоматически присваивается одному из доступных неголосующих узлов. Переключение происходит незаметно для пользователя.

Количество голосующих узлов в кластере не настраивается и зависит только от общего количества инстансов. Если инстансов 1 или 2, то голосующий узел один. Если инстансов 3 или 4, то таких узлов три. Для кластеров с 5 или более инстансами — пять голосующих узлов.

## Configuration reference

> `--cfg <path>`
: Read configuration parameters from file (toml / yaml).
*env*: `PICODATA_CFG`
*default*: *none*

`--data-dir`
: Here the instance persists all of its data.
*env*: `PICODATA_DATA_DIR`
*default*: `.`

`--listen`
: Socket bind address.
*env*: `PICODATA_LISTEN`
*default*: `localhost:3301`

`--peer <[host][:port],...>`
: Address of other instance(s).
*env*: `PICODATA_PEER`
*default*: `localhost:3301`

`--advertise <[host][:port]>`
: Address the other instances should use to connect to this instance.
*env*: `PICODATA_ADVERTISE`
*default*: `%listen%`

`--cluster-id <name>`
: Name of the cluster. The instance will refuse to join the cluster with a different name.
*env*: `PICODATA_CLUSTER_ID`
*default*: `demo`

`--instance-id <name>`
: Name of the instance.
*env*: `PICODATA_INSTANCE_ID`
*default*: `i%raft_id%`, e.g. `i1`, `i42`, etc.

`--failure-domain <key=value,...>`
: Comma-separated list describing physical location of the server. Each domain is a key-value pair. Until max replicaset count is reached, picodata will avoid putting two instances into the same replicaset if at least one key of their failure domains has the same value. Instead, new replicasets will be created. Replicasets will be populated with instances from different failure domains until the desired replication factor is reached.
*env*: `PICODATA_FAILURE_DOMAIN`
*default*: *none*

`--group <name>`
: Name of the instance group. It's an error to run instance with a group changed.
*env*: `PICODATA_GROUP`
*default*: `default`

`--role <name,...>`
: Valid roles are `"storage"` and `"router"`.
*env*: `PICODATA_ROLE`
*default*: `storage,router`

`--init-replication-factor <number>`
: Total number of replicas (copies of data) for each replicaset in the current group. It's only accounted upon the group creation (adding the first instance in the group), and ignored aftwerwards.
*env*: `PICODATA_INIT_REPLICATION_FACTOR`
*default*: `1`

`--init-storage-weight <number>`
: Proportional capacity of current instance. Common for each instance in the current group. Only valid for instances with "storage" role enabled.
*env*: `PICODATA_INIT_STORAGE_WEIGHT`
*default*: `1`

`--max-replicaset-count`
: Maximum number of replicasets in the current group.
*env*: `PICODATA_MAX_REPLICASET_COUNT`
*default*: don't modify
:hammer_and_wrench: Если меньше текущего размера группы, то игнорируется. Если в текущей группе нет места, то ошибка. Если больше — настройка применяется к текущей группе репликасетов.
